import sys
from pathlib import Path
sys.path.append(str(Path(__file__).parent))
import combine_data

NLIDTC = combine_data.NLIDataToCsv()

def downloadData(snliCsv, multinliCsv, anliCsv, dataCsv):
    NLIDTC.setSnliCsv(snliCsv)
    NLIDTC.setMultinliCsv(multinliCsv)
    NLIDTC.setAnliCsv(anliCsv)
    NLIDTC.setDataCsv(dataCsv)
    NLIDTC.doAllCommands()

downloadData("./snli.csv", "./multinli.csv", "./anli.csv", "./data.csv")



from transformers import AutoTokenizer, AutoModelForSeq2SeqLM, AutoModelForCausalLM

class STQ:

    def __init__(self):
        self.MODELNAME = "ramsrigouthamg/t5_squad_v1"
        self.model = AutoModelForSeq2SeqLM.from_pretrained(self.MODELNAME)
        self.tokenizer = AutoTokenizer.from_pretrained(self.MODELNAME)
        # self.answertokenizer = AutoTokenizer.from_pretrained("EleutherAI/gpt-neo-2.7B")
        # self.answermodel = AutoModelForCausalLM.from_pretrained("EleutherAI/gpt-neo-2.7B")

    def getQuestion(self, forumAns, forumQues):
        mdl = self.model
        tknizer = self.tokenizer
        sentence = forumAns
        answer = forumQues
        text = "context: {} answer: {}".format(sentence,answer)
        # print (text)
        max_len = 256
        encoding = tknizer.encode_plus(text,max_length=max_len, pad_to_max_length=False,truncation=True, return_tensors="pt")
        input_ids, attention_mask = encoding["input_ids"], encoding["attention_mask"]
        outs = mdl.generate(input_ids=input_ids,
                                        attention_mask=attention_mask,
                                        early_stopping=True,
                                        num_beams=5,
                                        num_return_sequences=1,
                                        no_repeat_ngram_size=2,
                                        max_length=300)
        dec = [tknizer.decode(ids,skip_special_tokens=True) for ids in outs]
        Question = dec[0].replace("question:","")
        Question= Question.strip()
        return Question

    def getQuestionWithoutResponse(self, response):
        mdl = self.model
        tknizer = self.tokenizer
        sentence = response
        text = "context: {}".format(sentence)
        # print (text)
        max_len = 256
        encoding = tknizer.encode_plus(text,max_length=max_len, pad_to_max_length=False,truncation=True, return_tensors="pt")
        input_ids, attention_mask = encoding["input_ids"], encoding["attention_mask"]
        outs = mdl.generate(input_ids=input_ids,
                                        attention_mask=attention_mask,
                                        early_stopping=True,
                                        num_beams=5,
                                        num_return_sequences=1,
                                        no_repeat_ngram_size=2,
                                        max_length=300)
        dec = [tknizer.decode(ids,skip_special_tokens=True) for ids in outs]
        Question = dec[0].replace("question:","")
        Question= Question.strip()
        return Question
        



from datasets import load_dataset_builder, load_dataset, get_dataset_split_names, get_dataset_config_names
import pandas as pd
import sys
from pathlib import Path
sys.path.append(str(str(Path(__file__).parent.parent.parent.parent) + "/normalizer"))
import normalize

class NLIDataToCsv:

    def __init__(self, snliCsv="", multinliCsv="", anliCsv="", dataCsv="", premise=[], hypothesis=[], label=[]):
        self.snliCsv = snliCsv
        self.multinliCsv = multinliCsv
        self.anliCsv = anliCsv
        self.dataCsv = dataCsv
        self.premise = premise
        self.hypothesis = hypothesis
        self.label = label
        self.Normalizer = normalize.NormalizeTextClass()

    def normalize(self, text):
        self.Normalizer.setInputText(str(text))
        self.Normalizer.doAllCommands()
        return self.Normalizer.getNormalizedText()

    def setSnliCsv(self, snliCsv):
        self.snliCsv = snliCsv
    
    def setMultinliCsv(self, multinliCsv):
        self.multinliCsv = multinliCsv

    def setAnliCsv(self, anliCsv):
        self.anliCsv = anliCsv 

    def setDataCsv(self, dataCsv):
        self.dataCsv = dataCsv

    def snliToCsv(self):
        snli = load_dataset("snli")
        dfTest = pd.DataFrame(snli['test'])
        dfTest = dfTest.iloc[1: , :]
        dfTrain = pd.DataFrame(snli['train'])
        dfTrain = dfTrain.iloc[1: , :]
        dfVal = pd.DataFrame(snli['validation'])
        dfVal = dfVal.iloc[1:, :]
        df = pd.concat([dfTest, dfTrain, dfVal])
        premise = []
        for i in df['premise'].tolist():
            premise.append(self.normalize(str(i)))
        hypothesis = []
        for i in df['hypothesis'].tolist():
            hypothesis.append(self.normalize(str(i)))
        label = [] 
        for i in df['label'].tolist():
            label.append(int(i))
        self.premise += premise
        self.hypothesis += hypothesis
        self.label += label
        snliData = {
            'premise': premise,
            'hypothesis': hypothesis,
            'label': label
        }
        df = pd.DataFrame(snliData)
        print("snli")
        print("--------")
        print(df.head())
        df.to_csv(self.snliCsv, mode='a', index=False, header=True)

    def multinliToCsv(self):
        multinli = load_dataset("multi_nli")
        dfTrain = pd.DataFrame(multinli['train'])
        dfTrain = dfTrain.iloc[1:, :]
        dfValMat = pd.DataFrame(multinli['validation_matched'])
        dfValMat = dfValMat.iloc[1:, :]
        dfValMisMat = pd.DataFrame(multinli['validation_mismatched'])
        dfValMisMat = dfValMisMat.iloc[1:, :]
        df = pd.concat([dfTrain, dfValMat, dfValMisMat])
        #df = df.drop(columns=['promptID (int32)', 'pairID (string)', 'premise_binary_parse (string)', 'premise_parse (string)', 'hypothesis_binary_parse (string)', 'hypothesis_parse (string)', 'genre (string)'], axis=1)
        premise = []
        for i in df['premise'].tolist():
            premise.append(self.normalize(str(i)))
        hypothesis = []
        for i in df['hypothesis'].tolist():
            hypothesis.append(self.normalize(str(i)))
        label = []
        for i in df['label'].tolist():
            label.append(int(i))
        self.premise += premise
        self.hypothesis += hypothesis
        self.label += label
        multinliData = {
            'premise': premise,
            'hypothesis': hypothesis,
            'label': label
        }
        df = pd.DataFrame(multinliData)
        print("multinli")
        print("--------")
        print(df.head())
        df.to_csv(self.multinliCsv, mode='a', index=False, header=True)

    def anliToCsv(self):
        anli = load_dataset("anli")
        dfTrainR1 = pd.DataFrame(anli['train_r1'])
        dfTrainR1 = dfTrainR1.iloc[1:, :]
        dfDevR1 = pd.DataFrame(anli['dev_r1'])
        dfDevR1 = dfDevR1.iloc[1:, :]
        dfTestR1 = pd.DataFrame(anli['test_r1'])
        dfTestR1 = dfTestR1.iloc[1:, :]
        dfTrainR2 = pd.DataFrame(anli['train_r2'])
        dfTrainR2 = dfTrainR2.iloc[1:, :]
        dfDevR2 = pd.DataFrame(anli['dev_r2'])
        dfDevR2 = dfDevR2.iloc[1:, :]
        dfTestR2 = pd.DataFrame(anli['test_r2'])
        dfTestR2 = dfTestR2.iloc[1:, :]
        dfTrainR3 = pd.DataFrame(anli['train_r3'])
        dfTrainR3 = dfTrainR3.iloc[1:, :]
        dfDevR3 = pd.DataFrame(anli['dev_r3'])
        dfDevR3 = dfDevR3.iloc[1:, :]
        dfTestR3 = pd.DataFrame(anli['test_r3'])
        dfTestR3 = dfTestR3.iloc[1:, :]
        df = pd.concat([dfTestR1, dfTestR2, dfTestR3, dfTrainR1, dfTrainR2, dfTrainR3, dfDevR1, dfDevR2, dfDevR3])
        premise = []
        for i in df['premise'].tolist():
            premise.append(self.normalize(str(i)))
        hypothesis = []
        for i in df['hypothesis'].tolist():
            hypothesis.append(self.normalize(str(i)))
        label = []
        for i in df['label'].tolist():
            label.append(int(i))
        self.premise += premise
        self.hypothesis += hypothesis
        self.label += label
        anliData = {
            'premise': premise,
            'hypothesis': hypothesis,
            'label': label
        }
        df = pd.DataFrame(anliData)
        print("anli")
        print("--------")
        print(df.head())
        df.to_csv(self.anliCsv, mode='a', index=False, header=True)

    def allDataToCsv(self):
        data = {
            'premise': self.premise,
            'hypothesis': self.hypothesis,
            'label': self.label
        }
        df = pd.DataFrame(data)
        df.to_csv(self.dataCsv, mode='a', index=False, header=True)

    def doAllCommands(self):
        self.snliToCsv()
        self.multinliToCsv()
        self.anliToCsv()
        self.allDataToCsv()





from transformers import AutoModelForSeq2SeqLM, AutoTokenizer

class CBQA:

    def __init__(self):
        self.MODELNAME = "google/t5-xxl-ssm-tqa"
        self.model = AutoModelForSeq2SeqLM.from_pretrained(self.MODELNAME)
        self.tokenizer = AutoTokenizer.from_pretrained(self.MODELNAME)

    def setPrompt(self, prompt):
        self.prompt = prompt

    def getResult(self):
        input_ids = self.tokenizer(self.prompt, return_tensors="pt").input_ids
        gen_output = self.model.generate(input_ids)[0]
        return str(self.tokenizer.decode(gen_output, skip_special_tokens=True))




import requests
import urllib
from requests_html import HTML
from requests_html import HTMLSession
from urllib.request import urlopen, Request
from bs4 import BeautifulSoup
import nltk

class SearchLinkData:

    def __init__(self):
        self.url = "https://google.com/search?q="
        self.headers = {
            'User-agent':
            'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/106.0.0.0 Safari/537.36'
        }
        self.numOfRes = 10
        self.NTC = normalize.NormalizeTextClass()

    def get_source(self, url):
        try:
            session = HTMLSession()
            response = session.get(url)
            return response
        except requests.exceptions.RequestException as e:
            print(e)

    def setQues(self, ques):
        self.ques = str(ques)
        self.url += urllib.parse.quote_plus(self.ques)
        self.url += "&num="
        self.url += str(self.numOfRes)
        response = self.get_source(self.url)
        links = list(response.html.absolute_links)
        google_domains = ('https://www.google.', 
                          'https://google.', 
                          'https://webcache.googleusercontent.', 
                          'http://webcache.googleusercontent.', 
                          'https://policies.google.',
                          'https://support.google.',
                          'https://maps.google.')
        for url in links[:]:
            if url.startswith(google_domains):
                links.remove(url)
        self.links = links

    def getWebsiteData(self):
        self.textFromWeb = []
        for url in self.links:
            req = Request(
                url=url,
                headers={'User-Agent': 'Mozilla/5.0'}
            )
            try:
                html = urlopen(url).read()
                soup = BeautifulSoup(html, features="html.parser")
                for script in soup(["script", "style"]):
                    script.extract()   
                text = str(soup.get_text())
                # lines = (line.strip() for line in text.splitlines())
                # chunks = (phrase.strip() for line in lines for phrase in line.split("  "))
                # text = '\n'.join(chunk for chunk in chunks if chunk)
                # self.NTC.setInputText(str(text))
                # self.NTC.doAllCommands()
                # text = self.NTC.getNormalizedText()
                text = text.replace('\n','')
                text = text.replace('\t','')
                text = text.replace("\'", '')
                text = text.replace('"', '')
                text = text.replace("\\", '')
                self.textFromWeb.append(text)
            except urllib.error.HTTPError as e:
                continue

    def getSentences(self):
        self.sentences = []
        for texts in self.textFromWeb:
            try:
                nltk.download('punkt')
                sentences = nltk.sent_tokenize(texts)  
                for i in sentences:
                    self.sentences.append(str(i))
            except Exception as e:
                print(e)
        return self.sentences



contractionsDict = { "ain't": "are not", "'s":" is", "aren't": "are not", "can't": "cannot", "can't've": "cannot have", "‘cause": "because", "could've": "could have", "couldn't": "could not", "couldn't've": "could not have", "didn't": "did not", "doesn't": "does not", "don't": "do not", "hadn't": "had not", "hadn't've": "had not have", "hasn't": "has not", "haven't": "have not", "he'd": "he would", "he'd've": "he would have", "he'll": "he will", "he'll've": "he will have", "how'd": "how did", "how'd'y": "how do you", "how'll": "how will", "I'd": "I would", "I'd've": "I would have", "I'll": "I will", "I'll've": "I will have", "I'm": "I am", "I've": "I have", "isn't": "is not", "it'd": "it would", "it'd've": "it would have", "it'll": "it will", "it'll've": "it will have", "let's": "let us", "ma'am": "madam", "mayn't": "may not", "might've": "might have", "mightn't": "might not", "mightn't've": "might not have", "must've": "must have", "mustn't": "must not", "mustn't've": "must not have", "needn't": "need not", "needn't've": "need not have", "o'clock": "of the clock", "oughtn't": "ought not", "oughtn't've": "ought not have", "shan't": "shall not", "sha'n't": "shall not", "shan't've": "shall not have", "she'd": "she would", "she'd've": "she would have", "she'll": "she will", "she'll've": "she will have", "should've": "should have", "shouldn't": "should not", "shouldn't've": "should not have", "so've": "so have", "that'd": "that would", "that'd've": "that would have", "there'd": "there would", "there'd've": "there would have", "they'd": "they would", "they'd've": "they would have","they'll": "they will", "they'll've": "they will have", "they're": "they are", "they've": "they have", "to've": "to have", "wasn't": "was not", "we'd": "we would", "we'd've": "we would have", "we'll": "we will", "we'll've": "we will have", "we're": "we are", "we've": "we have", "weren't": "were not","what'll": "what will", "what'll've": "what will have", "what're": "what are", "what've": "what have", "when've": "when have", "where'd": "where did", "where've": "where have", "who'll": "who will", "who'll've": "who will have", "who've": "who have", "why've": "why have", "will've": "will have", "won't": "will not", "won't've": "will not have", "would've": "would have", "wouldn't": "would not", "wouldn't've": "would not have", "y'all": "you all", "y'all'd": "you all would", "y'all'd've": "you all would have", "y'all're": "you all are", "y'all've": "you all have", "you'd": "you would", "you'd've": "you would have", "you'll": "you will", "you'll've": "you will have", "you're": "you are", "you've": "you have"}



from transformers import pipeline

class OBQA:

    def __init__(self):
        self.MODELNAME = "mrm8488/longformer-base-4096-finetuned-squadv2"
        self.model = pipeline("question-answering", model=self.MODELNAME, tokenizer=self.MODELNAME)

    def setQuestionContext(self, question, context):
        self.question = question
        self.context = context

    def getResult(self):
        res = self.model(question = self.question, context = self.context)
        return str(res['answer'])



# sentence embedding via sbert's library along side linkBERT-large to enable a new unique model which will perform generally more accurately than either bert-large nli via word embedding

from sentence_transformers import SentenceTransformer, models

# imports
import math
from sentence_transformers import InputExample, losses
from sentence_transformers.cross_encoder import CrossEncoder
from sentence_transformers.cross_encoder.evaluation import CESoftmaxAccuracyEvaluator
from torch.utils.data import DataLoader
from sentence_transformers import evaluation
import pandas as pd
from pathlib import Path

class ML:

    def __init__(self, EVALSTEPS, NUMEPOCHS, WARMUPSTEPS, MODELPATH, csvPath=""):
        #self.MODELNAME = "facebook/bart-large-mnli"
        #self.EVALSTEPS = 50
        #self.NUMEPOCHS = 20
        #self.WARMUPSTEPS = 100
        #self.MODELPATH = "./download/"
        self.MODELNAME = "michiyasunaga/LinkBERT-large"
        self.EVALSTEPS = EVALSTEPS
        self.NUMEPOCHS = NUMEPOCHS
        self.WARMUPSTEPS = WARMUPSTEPS
        self.MODELPATH = MODELPATH
        self.csvPath = csvPath

    def setCsvPath(self, csvPath):
        self.csvPath = csvPath
        print("DONE WITH SETTING CSV PATH")

    def readCsv(self):
        self.df = pd.read_csv(self.csvPath)
        print("DONE WITH READING CSV PATH")

    def dataModified(self):
        self.int2label = {0: "contradiction", 1 : "neutral", 2: "entailment"}
        self.trainingData = []
        self.testingData = []
        self.premise = []
        self.hypothesis = []
        self.label = []
        i = 0
        for idx, row in self.df.iterrows():
            i += 1
            if (i % 100 == 0):
                self.testingData.append(InputExample(texts=[row['premise'], row['hypothesis']], label=row['label']))
            tmp = row["label"]
            if (not(tmp == 0 or tmp ==1 or tmp == 2)):
                continue
            self.trainingData.append(InputExample(texts=[row['premise'], row['hypothesis']], 
                                                  label=row['label']))
            self.premise.append(row['premise'])
            self.hypothesis.append(row['hypothesis'])
            self.label.append(row['label'])
        print("DONE WITH DATA MODIFICATION")

    def trainModel(self):
        self.model = CrossEncoder(self.MODELNAME, 
                                  num_labels=len(self.int2label))
        self.trainDataloader = DataLoader(self.trainingData, 
                                          shuffle=True, 
                                          batch_size=16)
        # crossentropy loss
        # self.trainLoss = losses.SoftmaxLoss(model=self.model, 
        #                                     sentence_embedding_dimension=self.model.get_sentence_embedding_dimension(), 
        #                                     num_labels=3)
        # self.evaluator = evaluation.EmbeddingSimilarityEvaluator(self.premise, 
        #                                                          self.hypothesis, 
        #                                                          self.label)
        self.evaluator = CESoftmaxAccuracyEvaluator.from_input_examples(self.testingData, name='alotnli-dev')
        # self.model.fit(train_objectives=[(self.trainDataloader, self.trainLoss)],
        #           evaluator=self.evaluator,
        #           evaluation_steps=self.EVALSTEPS,
        #           epochs=self.NUMEPOCHS,
        #           warmup_steps=self.WARMUPSTEPS,
        #           output_path=self.MODELPATH)
        print("GETTING READY TO TRAIN")
        warmup_steps = math.ceil(len(self.trainDataloader) * self.NUMEPOCHS * 0.1) #10% of train data for warm-up
        self.model.fit(train_dataloader=self.trainDataloader,
                  evaluator=self.evaluator,
                  evaluation_steps=self.EVALSTEPS,
                  epochs=self.NUMEPOCHS,
                  warmup_steps=warmup_steps,
                  output_path=self.MODELPATH)
        print("DONE TRAINING")

NLI = ML(EVALSTEPS=50, NUMEPOCHS=20, WARMUPSTEPS=100, MODELPATH="./download/")
def main():
    NLI.setCsvPath("https://files.thamognya.com/projects/internet-nlp/data.csv")
    NLI.readCsv()
    NLI.dataModified()
    NLI.trainModel()

main()



import string
import contractions
import tokenizers
from tokenizers.normalizers import Lowercase, NFKD, StripAccents
import re
import sys
from pathlib import Path
sys.path.append(str(Path(__file__).parent.parent))
import constants

class NormalizeTextClass:

    def __init__(self, inputText = "", outputText=""):
        self.inputText = inputText
        self.outputText = outputText
        self.normalierSeq = tokenizers.normalizers.Sequence([Lowercase(), NFKD(), StripAccents()])
        for i in constants.contractionsDict:
            contractions.add(i, constants.contractionsDict[i])

    def setInputText(self, inputTextIn):
        self.inputText = str(inputTextIn)
        self.outputText = str(inputTextIn)

    def expandContradictions(self):
        self.outputText = contractions.fix(self.inputText)

    def normalize(self):
        self.outputText = self.normalierSeq.normalize_str(self.outputText)

    def removePunctuation(self):
        # Remove punctuation from sentence
        tmp = self.outputText.translate(str.maketrans('', '', string.punctuation))
        tmp = re.sub(r'[^\w\s]', '', tmp)
        tmp = tmp.replace('', '')
        tmp = tmp.lower()
        self.outputText = tmp

    def doAllCommands(self):
        self.expandContradictions()
        self.removePunctuation()
        self.normalize()
        
    def getNormalizedText(self):
        return self.outputText


"""
normalier = NormalizeTextClass()
testingInput = "It would be unfair to demand that people cease pirating files when those same people aren't paid for their participation in very lucrative network schemes. Ordinary people are relentlessly spied on, and not compensated for information taken from them. While I'd like to see everyone eventually pay for music and the like, I'd not ask for it until there's reciprocity."
normalier.setInputText(testingInput)
print(normalier.getNormalizedText())
"""



from setuptools import setup, find_packages
try:
    from pip._internal.req import parse_requirements
except ImportError:
    from pip.req import parse_requirements

with open("README.md", mode="r", encoding="utf-8") as readme_file:
    readme = readme_file.read()

setup(
    name="internet-nlp",
    version="0.0.1",
    author="Thamognya Kodi",
    author_email="contact@thamognya.com",
    description="Allowing NLPs to connect to the internet",
    long_description=readme,
    long_description_content_type="text/markdown",
    license="GNU Affero General Public License v3.0 or later",
    url="https://github.com/thamognya/internet-nlp",
    download_url="https://github.com/thamognya/internet-nlp",
    packages=find_packages(),
    python_requires=">=3.9.0",
    install_reqs = parse_requirements('requirements.txt', session='hack'),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)",
        "Programming Language :: Python :: 3.10",
        "Topic :: Scientific/Engineering :: Artificial Intelligence"
    ],
    keywords="Transformer NLP NLI Internet Networks BERT SOTA XLNet sentence embedding PyTorch NLP deep learning Tensorflow Huggingface"
)



from sentence_transformers import CrossEncoder

class NLI:

    def __init__(self):
        self.model = CrossEncoder('cross-encoder/nli-deberta-v3-large')

    def setPremiseHypothesis(self, premise, hypothesis):
        self.scores = self.model.predict([(premise, hypothesis)])

    def getLabel(self):
        label_mapping = ['contradiction', 'entailment', 'neutral']
        labels = [label_mapping[score_max] for score_max in self.scores.argmax(axis=1)]
        return labels[0]
